#include "MainWindow.h"
#include "ui_MainWindow.h"
#include "AboutWindow.h"
#include <QFileDialog>
#include <QMessageBox>
#include <limits>
#include <iostream>
#include <cmath>
#include "Plot.h"


MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow),
    originalImage(NULL),
    changedImage(NULL),
    originalPixmap(NULL)
{
    ui->setupUi(this);

    red1Plot = new Plot(this, "R", std::vector<int>(), Qt::red);
    green1Plot = new Plot(this, "G", std::vector<int>(), Qt::green);
    blue1Plot = new Plot(this, "B", std::vector<int>(), Qt::blue);
    histogram1Plot = new Plot(this, "I (HSI)", std::vector<int>(), Qt::black);
    cdf1Plot = new Plot(this, "Dystrybuanta", std::vector<int>(), Qt::white);

    ui->threeOriginalLayout->addWidget(red1Plot);
    ui->threeOriginalLayout->addWidget(green1Plot);
    ui->threeOriginalLayout->addWidget(blue1Plot);
    ui->originalHistLayout->addWidget(histogram1Plot);
    ui->originalCDFLayout->addWidget(cdf1Plot);

    red3Plot = new Plot(this, "R", std::vector<int>(), Qt::red);
    green3Plot = new Plot(this, "G", std::vector<int>(), Qt::green);
    blue3Plot = new Plot(this, "B", std::vector<int>(), Qt::blue);
    histogram3Plot = new Plot(this, "I (HSI)", std::vector<int>(), Qt::black);
    cdf3Plot = new Plot(this, "Dystrybuanta", std::vector<int>(), Qt::white);

    ui->threeChangedLayout->addWidget(red3Plot);
    ui->threeChangedLayout->addWidget(green3Plot);
    ui->threeChangedLayout->addWidget(blue3Plot);
    ui->changedHistLayout->addWidget(histogram3Plot);
    ui->changedCDFLayout->addWidget(cdf3Plot);

    histogram2Plot = new Plot(this, "Rysuj", std::vector<int>(), Qt::black);
    cdf2Plot = new Plot(this, "Dystrybuanta", std::vector<int>(), Qt::white);
    ui->drawingHistLayout->addWidget(histogram2Plot);
    ui->drawingCDFLayout->addWidget(cdf2Plot);

/*
    HSIPixel pix;
    pix.h = 240 * M_PI/180.0;
    pix.s = 1;
    pix.i = 240;
    //RGBPixel rgb;
    //rgb.r = 240;
    //rgb.g = 180;
    //rgb.b = 255;
    //RGBPixel rgb2 = hsi2rgb(rgb2hsi(rgb));
    //HSIPixel hsi = rgb2hsi(rgb);
    RGBPixel rgb = hsi2rgb(pix);
    std::cout << rgb.r << " " << rgb.g << " " << rgb.b << std::endl;

    //std::cout << rgb2.r << " " << rgb2.g << " " << rgb2.b << std::endl;
    //exit(0);
    //std::cout << hsi.h*180/M_PI << " " << hsi.s << " " << hsi.i << std::endl;*/
}


void MainWindow::Load(const QString& fileName)
{
    Clean();

    originalPixmap = new QPixmap(fileName);
    ui->originImageLabel->setPixmap(*originalPixmap);
    originalImage = new QImage(originalPixmap->toImage());


    RGBClass rgbImage = GetHistogramRGB(*originalImage);

    CreateImageArrays(originalImage);

    histogram1 = GetHistogram(hsiImage);
    cdf1 = GetCDF(histogram1);

    // nowe
    red1Plot->setData(rgbImage.r);
    green1Plot->setData(rgbImage.g);
    blue1Plot->setData(rgbImage.b);
    histogram1Plot->setData(histogram1);
    cdf1Plot->setData(cdf1);

    histogram2Plot->setAxisScale(QwtPlot::yLeft, 0, histogram1Plot->axisScaleDiv(0)->range());

}


void MainWindow::Clean()
{
    rgbImage.clear();
    hsiImage.clear();
    hsiImage2.clear();

    histogram1.clear();
    histogram2.clear();
    histogram3.clear();
    cdf1.clear();
    cdf2.clear();
    cdf3.clear();

    histogram1Plot->Clean();
    histogram2Plot->Clean();
    histogram3Plot->Clean();
    cdf1Plot->Clean();
    cdf2Plot->Clean();
    cdf3Plot->Clean();
    red1Plot->Clean();
    green1Plot->Clean();
    blue1Plot->Clean();
    red3Plot->Clean();
    green3Plot->Clean();
    blue3Plot->Clean();

    if (originalImage != NULL)
    {
        delete originalImage;
        originalImage = NULL;
    }
    if (originalPixmap != NULL)
    {
        ui->changedImageLabel->clear();
        delete originalPixmap;
        originalPixmap = NULL;
    }
    if (changedImage != NULL)
    {
        delete changedImage;
        changedImage = NULL;
    }
}

std::vector<int> MainWindow::GetCDF(const std::vector<int>& hist)
{
    std::vector<int> cdf(256);

    cdf[0] = hist[0];
    for (int i = 1; i < 256; ++i)
    {
        cdf[i] = cdf[i-1] + hist[i];
    }

    return cdf;
}


static inline double min3(double a, double b, double c)
{
    return std::min(a, std::min(b, c));
}

RGBPixel MainWindow::hsi2rgb(const HSIPixel& pix)
{
    RGBPixel rgb;
    const double H = pix.h;
    const double S = pix.s;
    const double I = static_cast<double>(pix.i);

    if (H >= 0.0 && H < 2*M_PI/3)
    {
        rgb.r = (1.0 + (S*cos(H)) / cos(M_PI/3-H)) * I;
        rgb.g = (1.0 + S*(1.0 - cos(H) / cos(M_PI/3-H))) * I;
        rgb.b = (1.0 - S) * I;
    }
    else if (H >= 2*M_PI/3 && H < 4*M_PI/3)
    {
        rgb.r = (1.0 - S) * I;
        rgb.g = (1.0 + (S*cos(H-2*M_PI/3)) / cos(M_PI-H)) * I;
        rgb.b = (1.0 + S*(1.0 - cos(H-2*M_PI/3) / cos(M_PI-H))) * I;
    }
    else if (H >= 4*M_PI/3 && H < 2*M_PI)
    {
        rgb.r = (1.0 + S*(1.0 - cos(H-4*M_PI/3) / cos(5*M_PI/3-H))) * I;
        rgb.g = (1.0 - S) * I;
        rgb.b = (1.0 + (S*cos(H-4*M_PI/3)) / cos(5*M_PI/3-H)) * I;
    }
    else
    {
        std::cout << "H poza zakresem " << std::endl;
        exit(0);
    }

    if (rgb.r > 255) rgb.r = 255;
    if (rgb.g > 255) rgb.g = 255;
    if (rgb.b > 255) rgb.b = 255;

    return rgb;
}


HSIPixel MainWindow::rgb2hsi(const RGBPixel& pix)
{
    HSIPixel hsi;
    const double r = static_cast<double>(pix.r);
    const double g = static_cast<double>(pix.g);
    const double b = static_cast<double>(pix.b);

    hsi.i = static_cast<int>((r + g + b) / 3.0);

    if (hsi.i == 0)
    {
        hsi.s = 0.0;
    }
    else
    {
        hsi.s = 1.0 - min3(r, g, b) / static_cast<double>(hsi.i);
    }

    if (std::abs(hsi.s) < std::numeric_limits<double>::epsilon())
    {
        hsi.h = 0.0;
    }
    else
    {
        hsi.h = acos((0.5*(r-g+r-b)) / (sqrt((r-g)*(r-g) + (r-b)*(g-b))));
        if (b > g)
            hsi.h = 2*M_PI - hsi.h;
    }
    return hsi;
}

void MainWindow::CreateImageArrays(const QImage* image)
{
    QRgb value;
    RGBPixel rgbPix;

    for (int i = 0; i < image->width(); i++)
    {
        for (int j = 0; j < image->height(); j++)
        {
            value = image->pixel(i, j);
            rgbPix.r = qRed(value);
            rgbPix.g = qGreen(value);
            rgbPix.b = qBlue(value);

            rgbImage.push_back(rgbPix);
            hsiImage.push_back(rgb2hsi(rgbPix));
        }
    }
}

// tylko intensywnosc
std::vector<int> MainWindow::GetHistogram(const std::vector<HSIPixel>& im)
{
    std::vector<int> histogram(256, 0);

    for (unsigned int i = 0; i < im.size(); ++i)
    {
        histogram[im[i].i]++;
    }

    return histogram;
}


RGBClass MainWindow::GetHistogramRGB(const QImage& image)
{
    RGBClass rgbImage;
    QRgb value;

    for (int i = 0; i < image.width(); i++)
    {
        for (int j = 0; j < image.height(); j++)
        {
            value = image.pixel(i, j);

            rgbImage.r[qRed(value)]++;
            rgbImage.g[qGreen(value)]++;
            rgbImage.b[qBlue(value)]++;
        }
    }
    return rgbImage;
}


void MainWindow::SaveImage()
{
    changedImage = new QImage(originalImage->width(), originalImage->height(), QImage::Format_RGB32);
    int n = 0;
    RGBPixel pix;

    for (int i = 0; i < changedImage->width(); i++)
    {
        for (int j = 0; j < changedImage->height(); j++)
        {
            pix = hsi2rgb(hsiImage2[n++]);
            changedImage->setPixel(i, j, qRgb(pix.r, pix.g, pix.b));
        }
    }

    ui->changedImageLabel->setPixmap(QPixmap::fromImage(*changedImage));
}

MainWindow::~MainWindow()
{
    delete ui;
    delete red1Plot;
    delete green1Plot;
    delete blue1Plot;
    delete histogram1Plot;
    delete histogram2Plot;
    delete histogram3Plot;
    delete cdf1Plot;
    delete cdf2Plot;
    delete cdf3Plot;
    delete red3Plot;
    delete green3Plot;
    delete blue3Plot;

    //Clean();
}


void MainWindow::on_actionExit_triggered()
{
    close();
}

void MainWindow::on_actionAbout_triggered()
{
    AboutWindow *aboutWin = new AboutWindow();
    aboutWin->show();
}



void MainWindow::on_actionOpen_triggered()
{
    QString fileName = QFileDialog::getOpenFileName(this, QString::fromUtf8("Otwórz plik"), ".", tr("Image files (*.bmp *.tif *.png)"));

    if (fileName != "")
    {
        Load(fileName);
    }
}

void MainWindow::on_okButton_clicked()
{

    histogram2 = histogram2Plot->GetPoints();

    // przeskalowanie histogramu
    int suma = 0;
    for (int i = 0; i < 256; i++)
        suma += histogram2[i];

    int i = 0;
    int allPoints = originalImage->width() * originalImage->height();
    if (suma < allPoints)
    {
        while (suma < allPoints)
        {
            if (i == 256) i = 0;
            ++histogram2[i++];
            ++suma;
        }
    }
    else if (suma > allPoints)
    {
        while (suma > allPoints)
        {
            if (i == 256) i = 0;
            if (histogram2[i] > 0)
            {
                --histogram2[i];
                --suma;
            }
            ++i;
        }
    }

    cdf2 = GetCDF(histogram2);

    histogram2Plot->setData(histogram2);
    cdf2Plot->setData(cdf2);

    hsiImage2 = hsiImage;

    // sporzadzenie tablicy LU (LUT)
    std::vector<int> lut(256);
    for (int i = 0; i < 255; ++i)
    {
        lut[i] = Check(cdf2, cdf1[i]);
    }

    for (unsigned int i = 0; i < hsiImage.size(); ++i)
    {
        hsiImage2[i].i = lut[hsiImage2[i].i];
    }

    // rysuj histogram
    histogram3 = GetHistogram(hsiImage2);
    cdf3 = GetCDF(histogram3);

    histogram3Plot->setData(histogram3);
    cdf3Plot->setData(cdf3);

    SaveImage();

    RGBClass rgbImage = GetHistogramRGB(*changedImage);
    red3Plot->setData(rgbImage.r);
    green3Plot->setData(rgbImage.g);
    blue3Plot->setData(rgbImage.b);
}


// sprawdz, dla jakiego x, wartosc y jest rowna cd
int MainWindow::Check(const std::vector<int>& cdf, int cd)
{
    for (int j = 0; j < 255; ++j)
    {
        if ((cd == cdf[j]) || (cd > cdf[j] && cd < cdf[j+1]))
        {
            return j;
        }
    }

    if (cd < cdf[0])
        return 0;

    if (cd >= cdf[255])
        return 255;

    std::cerr << "Nie znaleziono " << cd << std::endl;
    exit(0);
    return -1;
}


void MainWindow::on_cleanButton_clicked()
{
    histogram2Plot->Clean();
}

void MainWindow::on_actionSaveImage_triggered()
{
  //  if (solver->GetProjectDir() != "")
       {
        QString filename = QFileDialog::getSaveFileName(this, QString::fromUtf8("Zapisz plik"), QString("."), tr("TIF Files (*.tif)"));
           if( !filename.endsWith(".tif"))
           {
               filename.append(".tif");
           }

           if (!changedImage->save(filename, "TIF"))
           {
               QMessageBox::warning( this, QString::fromUtf8("Zapisywanie pliku"), QString::fromUtf8("Blad zapisu do katalogu : ") + filename);
           }
       }
    /*   else
       {

       }*/
}
