#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <vector>
#include "Plot.h"


namespace Ui {
    class MainWindow;
}

class RGBClass
{
public:
    RGBClass()
    {
        r.resize(256);
        g.resize(256);
        b.resize(256);
        for (int i = 0; i < 256; ++i) r[i] = g[i] = b[i] = 0;
    }

    std::vector<int> r, g, b;
};


class RGBPixel
{
public:
    RGBPixel() { r = g = b = 0;}
    int r, g, b;
};

class HSIPixel
{
public:
    HSIPixel() { h = s = i = 0;}
    int i;
    double h, s;
};


class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

private:
    Ui::MainWindow *ui;
    std::vector<int> histogram1;
    std::vector<int> histogram2;
    std::vector<int> histogram3;
    std::vector<int> cdf1;
    std::vector<int> cdf2;
    std::vector<int> cdf3;
    QImage *originalImage, *changedImage;
    QPixmap* originalPixmap;
    Plot *red1Plot, *green1Plot, *blue1Plot, ;
    Plot *histogram1Plot, *histogram2Plot, *histogram3Plot;
    Plot *red3Plot, *green3Plot, *blue3Plot;
    Plot *cdf1Plot, *cdf2Plot, *cdf3Plot;
    std::vector<RGBPixel> rgbImage;
    std::vector<HSIPixel> hsiImage;
    std::vector<HSIPixel> hsiImage2;

    void Load(const QString& fileName);
    void Clean();
    void CreateImageArrays(const QImage* image);
    std::vector<int> GetHistogram(const std::vector<HSIPixel>& im);
    int Check(const std::vector<int>& cdf, int cd);
    RGBClass GetHistogramRGB(const QImage& image);
    void SaveImage();
    std::vector<int> GetCDF(const std::vector<int>& hist);
    RGBPixel hsi2rgb(const HSIPixel& pix);
    HSIPixel rgb2hsi(const RGBPixel& pix);

private slots:
    void on_actionExit_triggered();
    void on_actionAbout_triggered();
    void on_actionOpen_triggered();
    void on_okButton_clicked();
    void on_cleanButton_clicked();
    void on_actionSaveImage_triggered();
};

#endif // MAINWINDOW_H
