#include "Plot.h"
#include <qwt_plot_curve.h>
#include <qwt_plot_marker.h>
#include <qwt_symbol.h>
#include <QMouseEvent>
#include <QMessageBox>
#include <iostream>


Plot::Plot(QWidget *parent, const QString& title, const std::vector<int>& col, const QColor& _color) :
    QwtPlot(parent),
    cSin(NULL)
{
    setAxisScale(QwtPlot::xBottom, 0, 255);
    setAxisTitle(QwtPlot::xBottom, title);
    color = _color;

    if (!col.empty())
    {
        setData(col);
    }
    point.resize(256);
    Clean();

    sym = new QwtSymbol(QwtSymbol::Ellipse, Qt::black, Qt::NoPen, QSize(3, 3));

}

Plot::~Plot()
{
    if (cSin != NULL)
    {
        cSin->setVisible(false);
        cSin->detach();
        delete cSin;
        cSin = NULL;
    }

    for (int i = 0; i < 256; ++i)
        point[i] = -11;

    for (unsigned int i = 0; i < m.size(); ++i)
    {
        m[i]->setVisible(false);
        m[i]->detach();
        //delete m[i];
        //m[i] = NULL;
    }

    m.clear();
}


void Plot::setData(const std::vector<int>& col)
{
    Clean();

    cSin = new QwtPlotCurve();
    cSin->setRenderHint(QwtPlotItem::RenderAntialiased);
    cSin->setLegendAttribute(QwtPlotCurve::LegendShowLine, true);
//   cSin->setPen(QPen(color));
    cSin->setBrush(QBrush(color));
    cSin->attach(this);

    for (int i = 0; i < 256; ++i)
    {
        x[i] = i;
        y[i] = col[i];
    }

    cSin->setSamples(x, y, 256);

    replot();
}


void Plot::mouseMoveEvent(QMouseEvent *event)
{

    if (event->buttons() & Qt::LeftButton)
    {
        QPointF pointClicked = event->pos();

        QPointF newPos = QPointF(invTransform(QwtPlot::xBottom, pointClicked.x())-23, invTransform(QwtPlot::yLeft, pointClicked.y()));
        int newSize = m.size();
        m.resize(newSize+1);
        m[newSize] = new QwtPlotMarker();
        m[newSize]->setLinePen(QPen(Qt::black, 0, Qt::DashDotLine));
        m[newSize]->setSymbol(sym);

        if (newPos.y() < 0)
            newPos.setY(0);
        if (newPos.y() > axisScaleDiv(0)->range())
            newPos.setY(axisScaleDiv(0)->range());

        if (newPos.x() < 0)
            newPos.setX(0);
        if (newPos.x() > 255)
            newPos.setX(255);

        m[newSize]->setValue(newPos);
        m[newSize]->attach(this);

        point[static_cast<int>(newPos.x())] = static_cast<int>(newPos.y());

        replot();
    }

}

void Plot::Clean()
{
    if (cSin != NULL)
    {
        cSin->setVisible(false);
        cSin->detach();
        delete cSin;
        cSin = NULL;
    }

    for (int i = 0; i < 256; ++i)
        point[i] = -11;

    for (unsigned int i = 0; i < m.size(); ++i)
    {
        m[i]->setVisible(false);
        m[i]->detach();
    }

    m.clear();

    replot();

}


std::vector<int> Plot::GetPoints()
{
    if (point[0] < 0)
        point[0] = 0;
    for (int i = 1; i < 256; ++i)
        if (point[i] < 0)
            point[i] = point[i-1];
    return point;
}
